#include "CRenderWindowLinux.h"

#ifdef _IRR_COMPILE_WITH_X11_DEVICE_


#include "CIrrDeviceLinux.h"
#include "os.h"



namespace irr {
namespace video {


CRenderWindowLinux::CRenderWindowLinux( Display *display, Window windowId , XVisualInfo* Visual, bool externalWindow ) :
    IRenderWindow(externalWindow),
    WindowHasFocus(false),
    WindowMinimized(false)
    , Resized(false)
    , StdHints(0)
    , screennr(0)
//    , visual(0)
//    , width(0)
//    , height(0)
    , X11Connection(display)
    , WindowId(windowId)
    , visual(Visual)
{

    screennr = DefaultScreen(X11Connection);

}





CRenderWindowLinux::~CRenderWindowLinux()
{


	if (!ExternalWindow)
	{
        forceDestroy();

	}


}



void CRenderWindowLinux::closeWindow(bool forceDestroy)
{
    XDestroyWindow(X11Connection, WindowId);

    if (visual)
        XFree(visual);
};


//SRenderWindowLinuxHandle
void*
CRenderWindowLinux::getWindowHandle()
{
//    SRenderWindowLinuxHandle VideoData;
    VideoData.X11Connection = X11Connection;
    VideoData.WindowId  = WindowId;
    VideoData.Visual = visual;
    return &VideoData;
}


// TODO to remove
void CRenderWindowLinux::forceDestroy()
{
    XDestroyWindow(X11Connection, WindowId);

    if (visual)
        XFree(visual);
}

core::dimension2d<u32> CRenderWindowLinux::getSize() const
{
    return core::dimension2d<u32> (0,0);
}


core::dimension2d<u32> CRenderWindowLinux::getPosition() const
{
    return core::dimension2d<u32> (0,0);
}


bool CRenderWindowLinux::isResizable() const
{
    return false;
}




//! sets the caption of the window
void CRenderWindowLinux::setWindowCaption(const wchar_t* text)
{
//#ifdef _IRR_COMPILE_WITH_X11_

	XTextProperty txt;
	if (Success==XwcTextListToTextProperty(X11Connection, const_cast<wchar_t**>(&text),
				1, XStdICCTextStyle, &txt))
	{
		XSetWMName(X11Connection, WindowId, &txt);
		XSetWMIconName(X11Connection, WindowId, &txt);
		XFree(txt.value);
	}
//#endif
}

//
////! presents a surface in the client area
////void* windowId,
//bool CRenderWindowLinux::present(video::IImage* image,  core::rect<s32>* srcRect)
//{
//#ifdef _IRR_COMPILE_WITH_X11_
//	// this is only necessary for software drivers.
//	if (!image)
//		return true;
////image
//
//	// thx to Nadav, who send me some clues of how to display the image
//	// to the X Server.
//
//	const u32 destwidth = SoftwareImage->width;
//	const u32 minWidth = core::min_(image->getDimension().Width, destwidth);
//	const u32 destPitch = SoftwareImage->bytes_per_line;
//
//	video::ECOLOR_FORMAT destColor;
//	switch (SoftwareImage->bits_per_pixel)
//	{
//		case 16:
//			if (SoftwareImage->depth==16)
//				destColor = video::ECF_R5G6B5;
//			else
//				destColor = video::ECF_A1R5G5B5;
//		break;
//		case 24: destColor = video::ECF_R8G8B8; break;
//		case 32: destColor = video::ECF_A8R8G8B8; break;
//		default:
//			os::Printer::log("Unsupported screen depth.");
//			return false;
//	}
//
//    // convert software image
//	u8* srcdata = reinterpret_cast<u8*>(image->lock());
//	u8* destData = reinterpret_cast<u8*>(SoftwareImage->data);
//
//	const u32 destheight = SoftwareImage->height;
//	const u32 srcheight = core::min_(image->getDimension().Height, destheight);
//	const u32 srcPitch = image->getPitch();
//	for (u32 y=0; y!=srcheight; ++y)
//	{
//		video::CColorConverter::convert_viaFormat(srcdata,image->getColorFormat(), minWidth, destData, destColor);
//		srcdata+=srcPitch;
//		destData+=destPitch;
//	}
//	image->unlock();
//
//	GC gc = DefaultGC(X11Connection, DefaultScreen(X11Connection));
////	Window myWindow=window;
////	if (windowId)
////		myWindow = reinterpret_cast<Window>(windowId);
//	XPutImage(X11Connection, Window, gc, SoftwareImage, 0, 0, 0, 0, destwidth, destheight);
//	return true;
//#endif
//	return false;
//}




//! returns if window is active. if not, nothing need to be drawn
bool CRenderWindowLinux::isWindowActive() const
{
	return (WindowHasFocus && !WindowMinimized);
}


//! returns if window has focus.
bool CRenderWindowLinux::isWindowFocused() const
{
	return WindowHasFocus;
}


//! returns if window is minimized.
bool CRenderWindowLinux::isWindowMinimized() const
{
	return WindowMinimized;
}


//! Minimize window
void CRenderWindowLinux::minimizeWindow()
{
#ifdef _IRR_COMPILE_WITH_X11_
	XIconifyWindow(X11Connection, WindowId, screennr);
#endif
}


//! Maximize window
void CRenderWindowLinux::maximizeWindow()
{
#ifdef _IRR_COMPILE_WITH_X11_
	XMapWindow(X11Connection, WindowId);
#endif
}


//! Restore original window size
void CRenderWindowLinux::restoreWindow()
{
#ifdef _IRR_COMPILE_WITH_X11_
	XMapWindow(X11Connection, WindowId);
#endif
}




//! Sets if the window should be resizable in windowed mode.
void CRenderWindowLinux::setResizable(bool resize)
{
//#ifdef _IRR_COMPILE_WITH_X11_
//	if (CreationParams.DriverType == video::EDT_NULL || CreationParams.Fullscreen )
//		return;
//
//	XUnmapWindow(X11Connection, WindowId);
//	if ( !resize )
//	{
//		// Must be heap memory because data size depends on X Server
//		XSizeHints *hints = XAllocSizeHints();
//		hints->flags=PSize|PMinSize|PMaxSize;
//		hints->min_width=hints->max_width=hints->base_width=Width;
//		hints->min_height=hints->max_height=hints->base_height=Height;
//		XSetWMNormalHints(X11Connection, WindowId, hints);
//		XFree(hints);
//	}
//	else
//	{
//		XSetWMNormalHints(X11Connection, WindowId, StdHints);
//	}
//	XMapWindow(X11Connection, WindowId);
//	XFlush(X11Connection);
//#endif // #ifdef _IRR_COMPILE_WITH_X11_
}


video::ECOLOR_FORMAT CRenderWindowLinux::getColorFormat() const
{
#ifdef _IRR_COMPILE_WITH_X11_
	if (visual && (visual->depth != 16))
		return video::ECF_R8G8B8;
	else
#endif
		return video::ECF_R5G6B5;

}

bool
CRenderWindowLinux::isFullscreen() const
{
    return false;
}



//! switches to fullscreen
bool CRenderWindowLinux::switchToFullScreen(bool reset)
{

    return false;
}



//! Notifies the device, that it has been resized
void CRenderWindowLinux::OnResized()
{
	Resized = true;
}





} // Namespace video
} // Namespace irr

 #endif //! _IRR_COMPILE_WITH_X11_DEVICE_
