#include "CRenderWindowWin32.h"
#include "CIrrDeviceWin32.h"
#include "os.h"


#ifdef _IRR_COMPILE_WITH_WINDOWS_DEVICE_




namespace irr {

namespace video {

// setPixelFormat(contextConfig,bool heuristic);
CRenderWindowWin32::CRenderWindowWin32(HWND windowHandle, bool fullscreen,bool externalWindow) :
    IRenderWindow(externalWindow)
    , IsNonNTWindows(false) //! TODO
    , WindowHandle(windowHandle)
    , IsFullscreen(fullscreen)
{

    RECT r;
    GetWindowRect(WindowHandle, &r);
    Config.WindowSize.Width = r.right - r.left;
    Config.WindowSize.Height = r.bottom - r.top;
    Config.Fullscreen = false;

    ShowWindow(WindowHandle, SW_SHOWNORMAL);
    UpdateWindow(WindowHandle);

    // fix ugly ATI driver bugs. Thanks to ariaci
    // TODO reenable
//    MoveWindow(WindowHandle, windowLeft, windowTop, realWidth, realHeight, TRUE);


	// set this as active window
	if ( WindowHandle )
	{
		SetActiveWindow(WindowHandle);
		SetForegroundWindow(WindowHandle);
	}
}




void
CRenderWindowWin32::closeWindow(bool forceDestroy)
{

	if (WindowHandle)
	{
		DestroyWindow(WindowHandle);

		WindowHandle = 0;
	}
}


void
CRenderWindowWin32::forceDestroy()
{


}


CRenderWindowWin32::~CRenderWindowWin32()
{

    if(!ExternalWindow)
        closeWindow();
}




// TODO to remove useless
//core::dimension2du CRenderWindowWin32::getBorderThickness()
//{
//    //!
//    core::dimension2du thickness;
////    if (!isFullscreen() )
////    {
////        if (resizable)
////        {
////            // thickness of horizontal border around window
////            thickness.Width = GetSystemMetrics(SM_CXSIZEFRAME);
////            // Height of caption area + thickness of vertical border
////            thickness.Y = GetSystemMetrics(SM_CYCAPTION) + GetSystemMetrics(SM_CYSIZEFRAME);
////        }
////        else
////        {
////            thickness.X = GetSystemMetrics(SM_CXDLGFRAME);
////            thickness.Y = GetSystemMetrics(SM_CYCAPTION) + GetSystemMetrics(SM_CYDLGFRAME);
////        }
////    }
//////    else
//////    {
//////        BorderX = BorderY = 0;
//////    }
//
//    return thickness;
//}



//! Notifies the device, that it has been resized
bool CRenderWindowWin32::resize(const core::dimension2du& newSize)
{




	char tmp[255];

	if (newSize.Width < 2 || newSize.Height < 2)
	{
//		sprintf(tmp, "Ignoring resize operation to (%ld %ld)", r.right, r.bottom);
		sprintf(tmp, "Ignoring resize operation to (%ud %ud)", newSize.Width, newSize.Height);
		os::Printer::log(tmp, ELL_INFORMATION);
		return false;
	}

    sprintf(tmp, "Resizing window to %uldx%ud", newSize.Width, newSize.Height);
    os::Printer::log(tmp);
//
//    RECT clientRect;
//    GetClientRect( WindowHandle, &clientRect);
//
//    LONG style = GetWindowLongPtr(WindowHandle,GWL_STYLE);
//	if(!SUCCEEDED(AdjustWindowRect(&clientRect, style, FALSE)) )
//    {
//
//    }
//
//   return MoveWindow( WindowHandle,
//              clientRect.left, clientRect.top,      //  top left corner
//              clientRect.right - clientRect.left,                // width
//              clientRect.bottom - clientRect.top,               // height
//              TRUE                  //allow the client area to be repainted.
//              ) == TRUE;

    // WORKING CODE
	RECT clientRect, windowRect;
    POINT ptDiff;

	GetClientRect( WindowHandle, &clientRect);
	GetWindowRect( WindowHandle, &windowRect);


   ptDiff.x = (windowRect.right - windowRect.left) - clientRect.right;
   ptDiff.y = (windowRect.bottom - windowRect.top) - clientRect.bottom;

   //!
   return MoveWindow( WindowHandle,
              windowRect.left, windowRect.top,      //  top left corner
              newSize.Width + ptDiff.x,                // width
              newSize.Height + ptDiff.y,               // height
              TRUE                  //allow the client area to be repainted.
              ) == TRUE;
}




//! sets the caption of the window
void CRenderWindowWin32::setWindowCaption(const wchar_t* text)
{
	DWORD dwResult;
	if (IsNonNTWindows)
	{
		const core::stringc s = text;
#if defined(_WIN64) || defined(WIN64)
		SetWindowTextA(WindowHandle, s.c_str());
#else
		SendMessageTimeout(WindowHandle, WM_SETTEXT, 0,
				reinterpret_cast<LPARAM>(s.c_str()),
				SMTO_ABORTIFHUNG, 2000, &dwResult);
#endif
	}
	else
	{
#if defined(_WIN64) || defined(WIN64)
		SetWindowTextW(WindowHandle, text);
#else
		SendMessageTimeoutW(WindowHandle, WM_SETTEXT, 0,
				reinterpret_cast<LPARAM>(text),
				SMTO_ABORTIFHUNG, 2000, &dwResult);
#endif
	}
}



//! returns if window is active. if not, nothing needs to be drawn
bool CRenderWindowWin32::isWindowActive() const
{
	_IRR_IMPLEMENT_MANAGED_MARSHALLING_BUGFIX;
	return (GetActiveWindow() == WindowHandle);
}





//! returns if window has focus
bool CRenderWindowWin32::isWindowFocused() const
{
	bool ret = (GetFocus() == WindowHandle);
	_IRR_IMPLEMENT_MANAGED_MARSHALLING_BUGFIX;
	return ret;
}


//! returns if window is minimized
bool CRenderWindowWin32::isWindowMinimized() const
{
	WINDOWPLACEMENT plc;
	plc.length=sizeof(WINDOWPLACEMENT);
	bool ret=false;
	if (GetWindowPlacement(WindowHandle,&plc))
		ret=(plc.showCmd & SW_SHOWMINIMIZED)!=0;
	_IRR_IMPLEMENT_MANAGED_MARSHALLING_BUGFIX;
	return ret;
}


//!
bool CRenderWindowWin32::isFullscreen() const
{
	_IRR_IMPLEMENT_MANAGED_MARSHALLING_BUGFIX;
	// TODO

//    LONG flags = GetWindowLong( WindowHandle, GWL_STYLE );
//
//    return (flags & WS_THICKFRAME);

//	DEVMODE dm;
//	memset(&dm, 0, sizeof(dm));
//	dm.dmSize = sizeof(dm);
//
//	// use default values from current setting
//	if(!EnumDisplaySettings(NULL, ENUM_CURRENT_SETTINGS, &dm) )
//    {
//        os::Printer::log("Wrong monitor",ELL_ERROR);
//        return false;
//    }
//
//
//
//
//	return dm.dmFields;

//IsFullscreen
    return IsFullscreen;
}


//! switches to fullscreen
// TODO switch from fullscreen to windowed mode ?
//bool reset
bool CRenderWindowWin32::setFullscreen(const bool& enable)
{
//	if (!CreationParams.Fullscreen)
//		return true;
//	if (reset)
//	{
//		if (ChangedToFullScreen)
//			return (ChangeDisplaySettings(NULL,0)==DISP_CHANGE_SUCCESSFUL);
//		else
//			return true;
//	}

    // <=> current state != enable
    if( !(enable  ^ isFullscreen()) )
    {
        os::Printer::log("Already set to the correct mode", ELL_INFORMATION);
        return true;
    }


    // go fullscreen
    if(enable)
    {


//
//     HMONITOR hmon = MonitorFromWindow(hwnd,
//                                       MONITOR_DEFAULTTONEAREST);
//     MONITORINFO mi = { sizeof(mi) };
//     if (!GetMonitorInfo(hmon, &mi)) return NULL;
//       mi.rcMonitor.left,
//       mi.rcMonitor.top,
//       mi.rcMonitor.right - mi.rcMonitor.left,
//       mi.rcMonitor.bottom - mi.rcMonitor.top,



    core::dimension2d<u32> windowSize = getClientSize();

	DEVMODE dm;
	memset(&dm, 0, sizeof(dm));
	dm.dmSize = sizeof(dm);

	// use default values from current setting
	EnumDisplaySettings(NULL, ENUM_CURRENT_SETTINGS, &dm);
	dm.dmPelsWidth = windowSize.Width;
	dm.dmPelsHeight = windowSize.Height;
//	dm.dmBitsPerPel = CreationParams.Bits;  //! todo get
//	dm.dmFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT | DM_DISPLAYFREQUENCY;
	dm.dmFields = DM_PELSWIDTH | DM_PELSHEIGHT | DM_DISPLAYFREQUENCY;

	LONG res = ChangeDisplaySettings(&dm, CDS_FULLSCREEN);
	if (res != DISP_CHANGE_SUCCESSFUL)
	{ // try again without forcing display frequency
		dm.dmFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;
		res = ChangeDisplaySettings(&dm, CDS_FULLSCREEN);
	}

	bool ret = false;
	switch(res)
	{
	case DISP_CHANGE_SUCCESSFUL:
//		ChangedToFullScreen = true;
		ret = true;
		break;
	case DISP_CHANGE_RESTART:
		os::Printer::log("Switch to fullscreen: The computer must be restarted in order for the graphics mode to work.", ELL_ERROR);
		break;
	case DISP_CHANGE_BADFLAGS:
		os::Printer::log("Switch to fullscreen: An invalid set of flags was passed in.", ELL_ERROR);
		break;
	case DISP_CHANGE_BADPARAM:
		os::Printer::log("Switch to fullscreen: An invalid parameter was passed in. This can include an invalid flag or combination of flags.", ELL_ERROR);
		break;
	case DISP_CHANGE_FAILED:
		os::Printer::log("Switch to fullscreen: The display driver failed the specified graphics mode.", ELL_ERROR);
		break;
	case DISP_CHANGE_BADMODE:
		os::Printer::log("Switch to fullscreen: The graphics mode is not supported.", ELL_ERROR);
		break;
	default:
		os::Printer::log("An unknown error occured while changing to fullscreen.", ELL_ERROR);
		break;
	}


		// store the location of the window
		GetWindowRect(WindowHandle,&OldWindowedRect);

		// switch off the title bar
	    DWORD dwstyle = GetWindowLong(WindowHandle, GWL_STYLE);
	    dwstyle &= ~WS_CAPTION;
	    SetWindowLong(WindowHandle, GWL_STYLE, dwstyle);

		// move the window to (0,0)
		SetWindowPos( WindowHandle, 0, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
		InvalidateRect( WindowHandle, 0, true);

		return true;

	}
	// go windowed mode
	else
    {
        // this resets the screen to the registry-stored values
        ChangeDisplaySettings(0, 0);

        // replace the title bar
        DWORD dwstyle = GetWindowLong(WindowHandle, GWL_STYLE);
        dwstyle = dwstyle | WS_CAPTION;
        SetWindowLong(WindowHandle, GWL_STYLE, dwstyle);

        // move the window back to its old position
        SetWindowPos(WindowHandle,
                    0,
//                    m_oldLoc.x,
//                    m_oldLoc.y,
                    10,10,
                    0, 0,
                    SWP_NOSIZE | SWP_NOZORDER
                    );
        InvalidateRect( WindowHandle, 0, true);
    }


	_IRR_IMPLEMENT_MANAGED_MARSHALLING_BUGFIX;
	return true;
}



core::dimension2d<u32>
CRenderWindowWin32::getPosition() const
{
    RECT windowRect;
    GetWindowRect(WindowHandle, &windowRect);
    core::dimension2d<u32> dims(windowRect.left, windowRect.top  );
    return dims;
}


core::dimension2d<u32>
CRenderWindowWin32::getClientSize() const
{
    RECT clientRect;
    GetClientRect(WindowHandle, &clientRect);
    core::dimension2d<u32> dims(clientRect.right - clientRect.left, clientRect.bottom - clientRect.top  );
    return dims;
}




bool CRenderWindowWin32::isResizable() const
{
    //! fct WindowFromDC
    LONG flags = GetWindowLong( WindowHandle, GWL_STYLE );

    return (flags & WS_THICKFRAME);
//    return false;
}



// Seems like there is a bug
//! Sets if the window should be resizable in windowed mode.
bool CRenderWindowWin32::setResizable(bool enable)
{

	RECT clientRect;
	RECT originalWindowRect;
	POINT ptDiff;

    if(isFullscreen())
    {
        os::Printer::log( "can't set as resizable a window in fullscreen mode", ELL_WARNING);
        return false;
    }

	LONG_PTR style = WS_POPUP;

	if (!enable)
		style = WS_SYSMENU | WS_BORDER | WS_CAPTION | WS_CLIPCHILDREN | WS_CLIPSIBLINGS;
	else
		style = WS_THICKFRAME | WS_SYSMENU | WS_CAPTION | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | WS_MINIMIZEBOX | WS_MAXIMIZEBOX;

	if (!SetWindowLongPtr(WindowHandle, GWL_STYLE, style))
    {


		os::Printer::log("Could not change window style.");
    }

	RECT clientSize;
	GetClientRect(WindowHandle,&clientSize);
//	clientSize.top = 0;
//	clientSize.left = 0;
//	clientSize.right = getVideoDriver()->getScreenSize().Width;
//	clientSize.bottom = getVideoDriver()->getScreenSize().Height;

	AdjustWindowRect(&clientSize, style, FALSE);

	const s32 realWidth = clientSize.right - clientSize.left;
	const s32 realHeight = clientSize.bottom - clientSize.top;

	const s32 windowLeft = (GetSystemMetrics(SM_CXSCREEN) - realWidth) / 2;
	const s32 windowTop = (GetSystemMetrics(SM_CYSCREEN) - realHeight) / 2;

	return SetWindowPos(WindowHandle, HWND_TOP, windowLeft, windowTop, realWidth, realHeight,
		SWP_FRAMECHANGED | SWP_NOMOVE | SWP_SHOWWINDOW) == TRUE;

}


video::ECOLOR_FORMAT CRenderWindowWin32::getColorFormat() const
{
	return video::ECF_R5G6B5;
}


//! Minimizes the window.
void CRenderWindowWin32::minimizeWindow()
{
	WINDOWPLACEMENT wndpl;
	wndpl.length = sizeof(WINDOWPLACEMENT);
	GetWindowPlacement(WindowHandle, &wndpl);
	wndpl.showCmd = SW_SHOWMINNOACTIVE;
	SetWindowPlacement(WindowHandle, &wndpl);
}


//! Maximizes the window.
void CRenderWindowWin32::maximizeWindow()
{
	WINDOWPLACEMENT wndpl;
	wndpl.length = sizeof(WINDOWPLACEMENT);
	GetWindowPlacement(WindowHandle, &wndpl);
	wndpl.showCmd = SW_SHOWMAXIMIZED;
	SetWindowPlacement(WindowHandle, &wndpl);
}


//! Restores the window to its original size.
void CRenderWindowWin32::restoreWindow()
{
	WINDOWPLACEMENT wndpl;
	wndpl.length = sizeof(WINDOWPLACEMENT);
	GetWindowPlacement(WindowHandle, &wndpl);
	wndpl.showCmd = SW_SHOWNORMAL;
	SetWindowPlacement(WindowHandle, &wndpl);
}


} // video
} // Namespace irr


#endif
